package parsing;

import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.StringTokenizer;
import java.io.FileInputStream;

import commons.*;
/**
* <h1>SimpleTextParser</h1>
* Responsible for handling the reading the
* data from a text file
*
* @version 1.0
* @since   2017-07-23
*/
public final class SimpleTextParser implements IParser {
	
	/**
	* {@inheritDoc}
	*/
	@Override
	public TimeLine parse (String fileName) throws FileNotFoundException {
		Scanner inputStream = null;
		TimeLine tl =  new TimeLine();
		
		//Catch the exception only where you can do sth about it;
		//Otherwise: throw it
		//if the file is not existent, this throws FileNotFoundException
		inputStream = new Scanner(new FileInputStream(fileName));		

		int numLinesParsed = 0;
		int numLinesWithErrors = 0;
		while (inputStream.hasNextLine()) {
			String line = inputStream.nextLine();
			numLinesParsed++;
			
			StringTokenizer tokenizer = new StringTokenizer(line);
			if(tokenizer.countTokens() != 3){
				numLinesWithErrors++;
				logErrorMsg(numLinesParsed, line);				
				continue;
			}
			String s = tokenizer.nextToken();
			int pos = -1;
			try{
				pos = Integer.parseInt(s);
			}
			catch(NumberFormatException e){
				numLinesWithErrors++;
				logErrorMsg(numLinesParsed, line);				
				continue;				
			}
				
			s = tokenizer.nextToken();
			double xValue = -1; 
			try {
				xValue = Double.parseDouble(s);
			}
			catch(NumberFormatException e){
				numLinesWithErrors++;
				logErrorMsg(numLinesParsed, line);				
				continue;				
			}
			
			s = tokenizer.nextToken();
			double yValue = -1;
			try {
				yValue = Double.parseDouble(s);
			}
			catch(NumberFormatException e){
				numLinesWithErrors++;
				logErrorMsg(numLinesParsed, line);				
				continue;				
			}
			
			ValuePair v = new ValuePair(pos,xValue,yValue);
			tl.addValue(v);		
		}
		System.out.println("Parsed #lines(total):\t" + numLinesParsed);
		System.out.println("Erroneous #lines:\t" + numLinesWithErrors);
		inputStream.close();

		return tl;
	}

	private void logErrorMsg(int numLinesParsed, String line) {
		System.err.println("Wrong Input format. I expect <pos xValue yValue> of type <int double double>. Line: " + numLinesParsed + "\n\t" + line);
	}

}//end class
