package engine;

import static org.junit.Assert.*;

import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

//import engine.MainEngine;


public class MainEngineTest {
	private MainEngine mainEngine;
	private int timelineSize;
	private int engineNumPhases; 
	
	@Before
	public void setUp() throws Exception {
		mainEngine = new MainEngine();
		engineNumPhases = 0;
		timelineSize = 0;
	}

	@Test
	public final void testSetTimeLineHappy() {
		timelineSize = mainEngine.setTimeLine("src/test/resources/input/smallTestTimeLine.txt");		
		int expectedTimeLineSize = 6;
		assertEquals(expectedTimeLineSize, timelineSize); 
	}
	
	@Test
	public final void testSetTimeLineNoInputFile() {
		int actualResult = mainEngine.setTimeLine("YouCantCatchMe.txt");		
		int expectedResult = -1;
		assertEquals(expectedResult, actualResult); 
	}
	

	@Test
	public final void testProducePhasesHappy() {
		mainEngine.setTimeLine("src/test/resources/input/smallTestTimeLine.txt");
		engineNumPhases = mainEngine.producePhases();
		int expectedNumPhases = 4;
		assertEquals(expectedNumPhases, engineNumPhases); 
		//fail("Not yet implemented"); 
	}

	@Test
	public final void testHtmlVisualize() {
//		char referenceRaster[][] = {{'v', '|', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '},
//		{'a', '|', ' ', ' ', ' ', ' ', ' ', ' ', '@', ' ', ' ', ' ', ' ', ' ', ' '},
//		{'l', '|', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '},
//		{'u', '|', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '},
//		{'e', '|', ' ', ' ', ' ', '@', ' ', '@', ' ', ' ', ' ', ' ', ' ', ' ', ' '},
//		{' ', '|', ' ', ' ', ' ', ' ', '@', ' ', '@', ' ', ' ', ' ', ' ', ' ', ' '},
//		{' ', '|', ' ', ' ', '@', ' ', ' ', ' ', ' ', '@', '@', ' ', ' ', ' ', ' '},
//		{' ', '|', '@', '@', ' ', ' ', ' ', ' ', ' ', ' ', ' ', '@', '@', ' ', ' '},
//		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', '@', ' '},
//		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '},
//		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '},
//		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '},
//		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '},
//		{'~', '~', '~', '~', '~', '~', '~', '~', '~', '~', '~', '~', '~', '~', '~'},
//		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', 'T', 'i', 'm', 'e'}};

		char referenceRaster[][] = {{'v', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{'a', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{'l', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{'u', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{'e', '|', ' ', ' ', ' ', '@', ' ', '@'},
		{' ', '|', ' ', ' ', ' ', ' ', '@', ' '},
		{' ', '|', ' ', ' ', '@', ' ', ' ', ' '},
		{' ', '|', '@', '@', ' ', ' ', ' ', ' '},
		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{'~', '~', '~', '~', '~', '~', '~', '~'},
		{' ', '|', ' ', ' ', 'T', 'i', 'm', 'e'}};

		timelineSize = mainEngine.setTimeLine("src/test/resources/input/smallTestTimeLine.txt");
		mainEngine.setVisualizer("HtmlVisualizer");
		char actualResult [][] = {};
		try {
			PrintStream printstream = new PrintStream("src/test/resources/output/resultHTMLVis.html");
			actualResult = mainEngine.visualize(printstream);
			
		} catch (FileNotFoundException e) {
			System.out.println("Problem opening file resultHMVis.html. Exiting now");
			System.exit(0);
		} 

		
		

//		for (int i=0; i< actualResult.length; i++) {
//			for(int j = 0; j < actualResult[0].length; j++ )
//				System.out.print(actualResult[i][j]);
//			System.out.println();
//		}
		Boolean testSuccess = true;
		for (int i=0; i< referenceRaster.length; i++)
			for(int j = 0; j < referenceRaster[0].length; j++ )
				if (actualResult[i][j] != referenceRaster[i][j])
					testSuccess = false;
		assertTrue(testSuccess);
	}

	
	@Test
	public final void testConsolelVisualize() {
		char referenceRaster[][] = {{'v', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{'a', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{'l', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{'u', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{'e', '|', ' ', ' ', ' ', '@', ' ', '@'},
		{' ', '|', ' ', ' ', ' ', ' ', '@', ' '},
		{' ', '|', ' ', ' ', '@', ' ', ' ', ' '},
		{' ', '|', '@', '@', ' ', ' ', ' ', ' '},
		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{' ', '|', ' ', ' ', ' ', ' ', ' ', ' '},
		{'~', '~', '~', '~', '~', '~', '~', '~'},
		{' ', '|', ' ', ' ', 'T', 'i', 'm', 'e'}};

		timelineSize = mainEngine.setTimeLine("src/test/resources/input/smallTestTimeLine.txt");
		mainEngine.setVisualizer("ConsoleVisualizer");
		char actualResult [][] = mainEngine.visualize(System.out);

		Boolean testSuccess = true;
		for (int i=0; i< referenceRaster.length; i++)
			for(int j = 0; j < referenceRaster[0].length; j++ )
				if (actualResult[i][j] != referenceRaster[i][j])
					testSuccess = false;
		assertTrue(testSuccess);
	}

	
	@Test
	public final void testReportPhases() {
		mainEngine.setTimeLine("src/test/resources/input/smallTestTimeLine.txt");
		engineNumPhases = mainEngine.producePhases();
		List<String> als = mainEngine.reportPhases();
		int reporterPhases = als.size();
		assertEquals(engineNumPhases, reporterPhases);
		//fail("Not yet implemented"); 
	}	
}//end class
