package parsing;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.FileNotFoundException;

import org.junit.Before;
import org.junit.Test;

import commons.TimeLine;
import commons.ValuePair;
//import parsing.IParser;
//import parsing.ParserFactory;
//import parsing.SimpleTextParser;
//import parsing.TestParser;



public class ParserTest {
	
	private ParserFactory parserFactory;
	
	@Before
	public void setUp() throws Exception {
		parserFactory = new ParserFactory();
	}

	@Test
	public void testParserSimpleParserCreation() {
		IParser parser = parserFactory.createParser("SimpleTextParser");
		assertNotNull(parser);
		assertTrue(parser instanceof SimpleTextParser);
	}
	@Test
	public void testParserDummyParserCreation() {
		IParser parser = parserFactory.createParser("TestParser");
		assertNotNull(parser);
		assertTrue(parser instanceof TestParser);
	}

	@Test
	public void testParserHappy() throws FileNotFoundException {
		IParser parser = parserFactory.createParser("SimpleTextParser");
		
		TimeLine tl = parser.parse("src/test/resources/input/smallTestTimeLine.txt");
		assertNotNull(tl);
		
		int expectedSize = 6;
		assertEquals(expectedSize,tl.getValues().size());
		
		for(ValuePair pair: tl.getValues()){
			assertNotEquals(-1, pair.getPos());
			assertNotEquals(-1, pair.getX());
			assertNotEquals(-1, pair.getY());
		}
	}
	
	@Test(expected=Exception.class)
	public void testParserNoFile() throws FileNotFoundException {
		IParser parser = parserFactory.createParser("SimpleTextParser");
		@SuppressWarnings("unused")
		TimeLine tl = parser.parse("thereisNobodyHome.txt");
	}
	
	@Test
	public void testParserHappyBrokenRecords() throws FileNotFoundException {
		IParser parser = parserFactory.createParser("SimpleTextParser");
		
		TimeLine tl = parser.parse("src/test/resources/input/erroneousTestTimeLine.txt");
		assertNotNull(tl);
		
		int expectedSize = 4;
		assertEquals(expectedSize,tl.getValues().size());
		
		for(ValuePair pair: tl.getValues()){
			assertNotEquals(-1, pair.getPos());
			assertNotEquals(-1, pair.getX());
			assertNotEquals(-1, pair.getY());
		}
	}
	
}
